@@ -16,7 +16,12 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 USA.*/
 
-/* Routines for quadratic programming and other constrained optimization. */
+/* Routines for quadratic programming and other constrained optimization.
+   NOTE: most of the ancient loops that look rather inefficient are better
+         optimized by the compiler than hand re-writing.
+	 It is unlikely that any tweaking short of a full BLAS/LAPACK
+	 re-write will get better efficiency.
+*/
 
 
 #include <stdlib.h>
@@ -39,17 +44,18 @@ USA.*/
 
 
 
-matrix addconQT(Q,T,a,u) matrix *Q,T,a,*u;
+matrix addconQT(matrix *Q,matrix T,matrix a,matrix *u)
 
 /* A constraint, a (a row vector), is added to the QT factorization of
    the working set. T must have been initialised square, and then had T.r
    set to correct length. */
 
 { int q,i,j;
-  double la,ra=0.0,*cV,*bV,*T1V;
+  double la,ra=0.0,*cV,*bV,*T1V,**QM;
   matrix b,c;
   c=initmat(Q->r,1);b=initmat(Q->r,1);(*u)=initmat(Q->r,1);
-  for (i=0;i<c.r;i++) for (j=0;j<a.c;j++) c.V[i]+=a.V[j]*Q->M[j][i];
+  QM = Q->M;
+  for (i=0;i<c.r;i++) for (j=0;j<a.c;j++) c.V[i]+=a.V[j]*QM[j][i];
   la=dot(c,c);
   cV=c.V;bV=b.V;
   q=T.c-T.r-1;
@@ -85,14 +91,14 @@ void GivensAddconQT(matrix *Q,matrix *T,matrix *a,matrix *s,matrix *c)
    */
 
 { int q,i,j;
-  double Qi,r,cc,ss,*bV,*sV,*cV,**QM,*QV,bb,bb1;
+  double Qi,r,cc,ss,*bV,*sV,*cV,*aV,**QM,*QV,bb,bb1;
   matrix b;
-  b.V=T->M[T->r]; b.r=Q->r;b.c=1;
-  for (i=0;i<T->c;i++) b.V[i]=0.0;
-  for (i=0;i<b.r;i++) for (j=0;j<Q->r;j++) b.V[i]+=Q->M[j][i]*a->V[j];
+  b.V=T->M[T->r]; b.r=Q->r;b.c=1;QM=Q->M; bV=b.V;aV=a->V;
+  for (i=0;i<T->c;i++) bV[i]=0.0;
+  for (i=0;i<b.r;i++) for (j=0;j<Q->r;j++) bV[i]+=QM[j][i]*aV[j];
   /* now calculate a series of Givens rotations that will rotate the null basis
      so that it is orthogonal to new constraint a */
-  bV=b.V;cV=c->V;sV=s->V;QM=Q->M;
+  cV=c->V;sV=s->V;
   q=T->c-T->r-1; /* number of Givens transformations needed */
   for (i=0;i<q;i++)
   { /* first calculate the Givens transformation */
@@ -100,7 +106,7 @@ void GivensAddconQT(matrix *Q,matrix *T,matrix *a,matrix *s,matrix *c)
     r=bb*bb+bb1*bb1;r=sqrt(r);
     if (r==0.0) { ss=sV[i]=0.0;cc=cV[i]=1.0;} else
     { ss=sV[i]=bb/r;cc=cV[i]= -bb1/r;
-      bV[i]=0.0; /* non-essential */
+      //bV[i]=0.0; /* non-essential */
       bV[i+1]=r;
     }
     /* now apply it to Q */
@@ -198,7 +204,7 @@ int LSQPstep(int *ignore,matrix *Ain,matrix *b,matrix *p1,matrix *p,matrix *pk)
     if (!ignore[i])     /* skip any already in working set */
     { Ap1=0.0;
       for (j=0;j<Ain->c;j++) Ap1+=AV[j]*p1V[j]; /* form  A p1 = A(p+pk) */
-      if ((b->V[i]-Ap1)>0.0) /* does p+pk violate the ith constraint? */
+      if ((b->V[i]-Ap1)>0) /* does p+pk violate the ith constraint? */
       { ap=0.0;apk=0.0;        /* working out quantities needed to find distance to constraint from p */
 	for (j=0;j<Ain->c;j++)
 	{ ap+=AV[j]*pV[j];
@@ -400,7 +406,7 @@ void QPCLS(matrix *Z,matrix *X, matrix *p, matrix *y,matrix *Ain,matrix *b,matri
       vectors and the indexing arrays, obtain Z, and return.
 
 
-   On exit active[] contains the number of active inequlity constraints in active[0], 
+   On exit active[] contains the number of active inequality constraints in active[0], 
    and the row number of these constraints in Ain in the remaining elements of
    active[], active must be initialized to length p.r+1 on entry.
 
@@ -419,7 +425,7 @@ void QPCLS(matrix *Z,matrix *X, matrix *p, matrix *y,matrix *Ain,matrix *b,matri
 */
 
 { matrix Q,T,Rf,PX,Py,a,P,p1,s,c,Xy,y1,u,Pd,pz,pk;
-  int k,i,j,tk,*I,*ignore,iter=0,*fixed,*delog,maxdel=100;
+  int k,i,j,tk,*I,*ignore,*fixed,*delog,maxdel=100;
   double x;
   I=(int *)CALLOC((size_t) p->r,sizeof(int)); /* I[i] is the row of Ain containing ith active constraint */
   fixed=(int *)CALLOC((size_t) p->r,sizeof(int)); /* fixed[i] is set to 1 when the corresponding inequality constraint is to be left in regardless of l.m. estimate */
@@ -455,10 +461,12 @@ void QPCLS(matrix *Z,matrix *X, matrix *p, matrix *y,matrix *Ain,matrix *b,matri
   P=initmat(b->r,1); /* used solely for feasibility checking */
   Pd=initmat(y->r,1);pz=initmat(p->r,1);pk=initmat(p->r,1);
   tk=0;             /* The number of inequality constraints currently active */
+  for (i=0;i<active[0];i++) { /* immediately add supplied active set to constraints */
+    k = active[i+1];ignore[k]=1;I[tk] = k;
+    LSQPaddcon(Ain,&Q,&T,&Rf,&Py,&PX,&s,&c,k);tk++;
+  }  
   /*printf("\nLSQ");*/
-  while(1)
-  { iter++;
-    /* Form Pd=Py-PXp and minimize ||R pz - Pd|| */
+  while(1) { /* Form Pd=Py-PXp and minimize ||R pz - Pd|| */
     vmult(&PX,p,&Pd,0); /* Pd = PXp */
     for (i=0;i<Pd.r;i++) Pd.V[i] = Py.V[i]-Pd.V[i]; /* Pd=P(y-Xp) */
     Rf.c=Rf.r=p->r-tk-Af->r; /* Restrict attention to QR factor of PXZ */
@@ -470,7 +478,7 @@ void QPCLS(matrix *Z,matrix *X, matrix *p, matrix *y,matrix *Ain,matrix *b,matri
     for (i=0;i<pk.r;i++)
     { pk.V[i]=0.0; for (j=0;j<pz.r;j++) pk.V[i]+=Q.M[i][j]*pz.V[j];}
     /* Take a step from p along pk to minimum or a constraint ... */
-    k=LSQPstep(ignore,Ain,b,&p1,p,&pk);   /* s is the constraint to include or -1 */
+    k=LSQPstep(ignore,Ain,b,&p1,p,&pk);   /* k is the constraint to include or -1 */
     mcopy(&p1,p); /* updating the parameter vector */
     if (k>-1) /* add a constraint to the working set and update Rf, Py and PX */
     { I[tk]=k;ignore[k]=1; /* keeping track of what's in working set */
@@ -512,7 +520,7 @@ void QPCLS(matrix *Z,matrix *X, matrix *p, matrix *y,matrix *Ain,matrix *b,matri
       }
     }
   }
-}
+} /* QPCLS */
 
 
 void PCLS(matrix *X,matrix *p,matrix *y,matrix *w,matrix *Ain,matrix *b,
@@ -547,7 +555,7 @@ void PCLS(matrix *X,matrix *p,matrix *y,matrix *w,matrix *Ain,matrix *b,
 
 { int i,j,k,n;
   matrix z,F,W,Z,B;
-  double x,xx,*p1,*C;
+  double *p1,*C;
  
   /* form transformed data vector z */
   if (m>0) z=initmat(y->r+p->r,1);else z=initmat(y->r,1);
@@ -576,12 +584,102 @@ void PCLS(matrix *X,matrix *p,matrix *y,matrix *w,matrix *Ain,matrix *b,
   QPCLS(&Z,&F,p,&z,Ain,b,Af,active); /* note that at present Z is full not HH */
   /* working out value of objective at minimum */
   B=initmat(z.r,1);matmult(B,F,*p,0,0);
-  xx=0.0;for (i=0;i<z.r;i++) { x=B.V[i]-z.V[i];xx+=x*x;}
-  /*printf("\nObjective at Minimum = %g\n",xx);*/ freemat(B);
+  /*xx=0.0;for (i=0;i<z.r;i++) { x=B.V[i]-z.V[i];xx+=x*x;}
+    printf("\nObjective at Minimum = %g\n",xx);*/
+  freemat(B);
   /* freeing storage .... */
   freemat(F);freemat(z);freemat(W);freemat(Z);
-}
+} /*PCLS*/
 
+void RUnpackSarray(int m,matrix *S,double *RS)
+/* unpacks the R array RS into an array of matrices initialized to the correct dimensions 
+   let kk = sum_{i=0}^k S[i].r*S[i].c
+   Then the kth matrix starts at element kk of RS and stops at element k(k+1)
+   ... let this extracted array be M. S[k].M[i][j]=M[i+S[k].r*j] - in this way we ensure that 
+   M can be extracted straight to a matrix in R with 
+   A<-matrix(M,S[k].r,S[k].c) 
+*/ 
+{ int start,i,j,k;
+  start=0;
+  for (k=0;k<m;k++)
+  { for (i=0;i<S[k].r;i++) for (j=0;j<S[k].c;j++) S[k].M[i][j]=RS[start+i+S[k].r*j];
+    start += S[k].r*S[k].c;
+  }
+} /* RUnpackSarray */
+
+
+void  RPCLS(double *Xd,double *pd,double *yd, double *wd,double *Aind,double *bd,
+            double *Afd,double *Sd,
+            int *off,int *dim,double *theta, int *m,int *nar,int *active)
+
+/* Interface routine for PCLS the constrained penalized weighted least squares solver.
+   nar is an array of dimensions. Let:
+   n=nar[0] - number of data
+   np=nar[1] - number of parameters
+   nai=nar[2] - number of inequality constraints
+   naf=nar[3] - number of fixed constraints
+   
+   
+   Problem to be solved is:
+
+   minimise      ||W^0.5 (y - Xp)||^2 + p'Bp
+   subject to    Ain p >= b  & Af p = "constant"
+
+   where B = \sum_{i=1}^m \theta_i S_i and W=diag(w)
+
+   - in fact S_i are not stored whole - rather the smallest non-zero sub-matrix of each S_i is 
+   stored in a densely packed form in S[]: see routines RpackSarray() and RUnpackSarray() for 
+   details of the sub-matrix packing. off[i],off[i] is the location within the full S_i to
+   insert the sub-matrix actually stored which is of dimension dim[i] by dim[i].
+
+   W = diag(w) 
+
+   on exit p contains the best fit parameter vector. 
+
+*/
+{ matrix y,X,p,w,Ain,Af,b,*S;
+  int n,np,i;
+ 
+  np=nar[1];n=nar[0];
+  /* unpack from R into matrices */
+  X=Rmatrix(Xd,(long)n,(long)np);
+  p=Rmatrix(pd,(long)np,1L);
+  y=Rmatrix(yd,(long)n,1L);
+  w=Rmatrix(wd,(long)n,1L);
+  if (nar[2]>0) Ain=Rmatrix(Aind,(long)nar[2],(long)np); else Ain.r=0L;
+  if (nar[3]>0) Af=Rmatrix(Afd,(long)nar[3],(long)np); else Af.r=0L;
+  if (nar[2]>0) b=Rmatrix(bd,(long)nar[2],1L);else b.r=0L;
+ 
+  if (*m) S=(matrix *)CALLOC((size_t) *m,sizeof(matrix));
+  else S=NULL; /* avoid spurious compiler warning */
+  for (i=0;i< *m;i++) S[i]=initmat((long)dim[i],(long)dim[i]);
+  RUnpackSarray(*m,S,Sd);
+  
+  //if (nar[4]) H=initmat(y.r,y.r); else H.r=H.c=0L;
+  //active=(int *)CALLOC((size_t)(p.r+1),sizeof(int)); /* array for active constraints at best fit active[0] will be  number of them */
+  /* call routine that actually does the work */
+ 
+  PCLS(&X,&p,&y,&w,&Ain,&b,&Af,S,off,theta,*m,active);
+
+  /* copy results back into R arrays */ 
+  for (i=0;i<p.r;i++) pd[i]=p.V[i];
+ 
+  //if (H.r) RArrayFromMatrix(Hd,H.r,&H);
+  /* clear up .... */
+  //FREE(active);
+ 
+  for (i=0;i< *m;i++) freemat(S[i]);
+  if (*m) FREE(S);
+ 
+  freemat(X);freemat(p);freemat(y);freemat(w);
+  //if (H.r) freemat(H);
+  if (Ain.r) freemat(Ain);
+  if (Af.r) freemat(Af);
+  if (b.r) freemat(b);
+  //#ifdef MEM_CHECK
+  //dmalloc_log_unfreed();  dmalloc_verify(NULL);
+  //#endif
+} /* RPCLS */
 
 
 /***************************************************************************/
