@@ -14,12 +14,13 @@
 #include <stdio.h>
 #include <math.h>
 
-#include "mgcv.h"
-#include <R.h>
-#ifdef OPENMP_ON
+#ifdef _OPENMP
 #include <omp.h>
 #endif
 
+#include "mgcv.h"
+#include <R.h>
+
 /* A sparse matrix structure useful for storing discrete sparse marginals */
 typedef struct {
   int m,c, // matrix is m by c
@@ -108,7 +109,6 @@ void cs_trans(int *Ap,int *Ai,double *Ax,int *Cp,int *Ci, double *Cx,int *w,int
 } /* cs_trans */  
 
 
-
 void sprealloc(spMat *A, int nzmax) {
 /* Alter the storage in A to accomodate at most nzmax non-zero entries. 
    Note that if using openMP then allocation of storage needs to be locked.
@@ -661,7 +661,7 @@ SEXP sXWXd(SEXP X,SEXP W,SEXP LT, SEXP RT,SEXP NT) {
     Xs[i].nk = ks[mx+i] - ks[i]; // number of index vectors for this term
     Xs[i].off = off + off_start[ks[i]]; // the .nk offset .m+1 vectors for .r 
     j = Xs[i].p[Xs[i].c]; /* number of NZs in this matrix */
-    if (j>nzmax) nzmax=j; /* largest amount of NZ storage required by any eleemnt of Xs */
+    if (j>nzmax) nzmax=j; /* largest amount of NZ storage required by any element of Xs */
     /* now Cache its transpose... */
     spalloc(Xt+i,Xs[i].m,j);
     cs_trans(Xs[i].p,Xs[i].i,Xs[i].x,Xt[i].p,Xt[i].i,Xt[i].x,iwork,Xs[i].m,Xs[i].c);
